/**
 * Cecilia, an implementation of the Fractal component model in C.
 *
 * Copyright (C) 2007 ST Microelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Contact: fractal@objectweb.org
 *
 * Authors: Matthieu Leclercq
 */
#include <stdio.h>

/** Declare component internal data */
DECLARE_DATA {
  int nrOfInvocations;
};

/** Include cecilia.h. Must be included after the DECLARE_DATA */
#include <cecilia.h>

// -----------------------------------------------------------------------------
// Implementation of the service interface.
// -----------------------------------------------------------------------------

// void print(string msg)
void METHOD(s, print)(void *_this, char *msg) 
{
  int i;
  char* header = ATTRIBUTES.header;
  if (header == NULL) {
    header = "";
  }
  
  printf("Server: begin printing...\n");
  for (i = 0; i < ATTRIBUTES.count; ++i) {
    printf("%s%s\n", header, msg);
  }
  
  // increment the number of invocations received by this component instance
  DATA.nrOfInvocations++;

  printf("Server: print done (Number of invocations up to now: #%d)\n", DATA.nrOfInvocations);
}

// -----------------------------------------------------------------------------
// Implementation of the lifecycle-controller interface.
// -----------------------------------------------------------------------------

// int startFc();
int METHOD(lifecycle_controller, startFc)(void* _this)
{
  // initialize number of invocations
  DATA.nrOfInvocations = 0;
  return fractal_api_ErrorConst_OK;
}

// int stopFc();
int METHOD(lifecycle_controller, stopFc)(void* _this)
{
  return fractal_api_ErrorConst_OK;
}

// int getFcState();
int METHOD(lifecycle_controller, getFcState)(void* _this)
{
  return fractal_api_ErrorConst_OPERATION_NOT_SUPPORTED;
}
