/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.adl.idl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.Import;
import org.objectweb.fractal.cecilia.adl.idl.ast.ImportContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.RecordDefinition;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds generic record definitions. The generated code can be used for
 * component attribute definitions and complex data structure definitions.
 */
public class CAttributeBuilder extends AbstractRecordVisitor
    implements
      IDLBuilder {
  String     recordName;
  CodeWriter cw = null;

  public Object createIDLDefinition(final Object componentType,
      final Object componentInstance, final IDLDefinition idlDefinition,
      final Map<String, Object> codePieces, final Map<Object, Object> context)
      throws Exception {
    cw = new CodeWriter("C Attribute builder");
    // Pre-visit operations
    this.recordName = idlDefinition.getName();
    if (recordName != null) {

      // Visit
      visit(idlDefinition);
      // Post-visit operations
      final File file = new File(context.get("idlBuildDirectory").toString(),
          idlDefinition.getName().replace('.', File.separatorChar) + ".idl.h");
      if (!file.getParentFile().exists()) file.getParentFile().mkdirs();

      final PrintStream out = new PrintStream(new FileOutputStream(file));
      out.print(cw.toString());
      out.close();
    }
    return cw.toString();
  }

  public void enterRecord(final RecordDefinition record) throws Exception {
    cw.append("/* Generated record :" + recordName + " */").endl();

    final String struct = recordName.replace('.', '_');
    cw.append("#ifndef _").append(struct.toUpperCase()).append("_IDL_H_")
        .endl();
    cw.append("#define _").append(struct.toUpperCase()).append("_IDL_H_")
        .endl();
    cw.endl();
    cw.endl();

    /* include the definition of the cecilia short cut types */
    cw.endl();
    cw.append("/* some cecilia typedefs for predefinite C types */").endl();
    cw.append("#include \"cecilia_types.h\"").endl();
    cw.endl();

    cw.endl();

    final Import[] imports = ((ImportContainer) record).getImports();
    for (final Import imp : imports) {
      cw.append(CUtil.buildInclude(imp.getName()));
    }

    cw.append("struct ").append(struct).append(";").endl();
    cw.append("typedef struct ").append(struct).append(" R").append(struct)
        .append(";").endl();
    cw.append("struct ").append(struct).append("{").endl();
  }

  public void leaveRecord(final RecordDefinition record) throws Exception {
    cw.append("};").endl();
    cw.endl();
    final String struct = recordName.replace('.', '_');
    cw.append("#define ").append(struct.toUpperCase()).append("_POINTER \\")
        .endl();
    cw.append("  struct ").append(struct).append("* ").endl();
    cw.append("#define ").append(struct.toUpperCase()).append("_STRUCTURE \\")
        .endl();
    cw.append("  struct ").append(struct).endl();

    cw.append("#endif").endl();
  }

  public void visitField(final Field field) throws Exception {
    cw.append(CUtil.buildRecordField(field));
    cw.append(";").endl();
  }
}
