/***
 * Cecilia ADL Compiler
 * Copyright (C) 2008 INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Alessio Pace
 */

package org.objectweb.fractal.cecilia.adl.idl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.idl.ast.EnumDefinition;
import org.objectweb.fractal.cecilia.adl.idl.ast.EnumNameValuePair;
import org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Generate a <code>C enum</code> type from the IDL {@link EnumDefinition} AST
 * Node.
 * 
 * @author Alessio Pace
 */
public class CEnumBuilder implements IDLBuilder {

  /*
   * (non-Javadoc)
   * 
   * @see org.objectweb.fractal.cecilia.adl.idl.IDLBuilder#createIDLDefinition(java.lang.Object,
   *      java.lang.Object,
   *      org.objectweb.fractal.cecilia.adl.idl.ast.IDLDefinition,
   *      java.util.Map, java.util.Map)
   */
  public Object createIDLDefinition(final Object componentType,
      final Object componentInstance, final IDLDefinition idlDefinition,
      final Map<String, Object> codePieces, final Map<Object, Object> context)
      throws Exception {

    /* cast the input IDLDefinition parameter */
    final EnumDefinition enumDefinition = (EnumDefinition) idlDefinition;

    final CodeWriter cw = new CodeWriter("C Enum builder");
    final String enumName = enumDefinition.getName();

    cw.append("/* Generated enum :" + enumName + " */").endl();

    final String enumCName = enumName.replace('.', '_');
    cw.append("#ifndef _").append(enumCName.toUpperCase()).append("_IDL_H_")
        .endl();
    cw.append("#define _").append(enumCName.toUpperCase()).append("_IDL_H_")
        .endl();
    cw.endl();
    cw.endl();

    cw.append("enum " + enumCName + " {").endl();
    int i = 1;
    final int nrOfPairs = enumDefinition.getEnumNameValuePairs().length;
    for (final EnumNameValuePair pair : enumDefinition.getEnumNameValuePairs()) {
      cw.append("  " + pair.getName());
      final String pairValue = pair.getValue();
      if (pairValue != null && pairValue.length() > 0) {
        cw.append("=" + pairValue);
      }

      if (i != nrOfPairs) {
        cw.append(",");
      }

      i++;
      cw.endl();
    }
    cw.append("};").endl().endl();

    /* typedef with "R" in front of the enum type name */
    cw.append("typedef enum ").append(enumCName).append(" R").append(enumCName)
        .append(";").endl();

    cw.append("#endif").endl();

    final File file = new File(context.get("idlBuildDirectory").toString(),
        idlDefinition.getName().replace('.', File.separatorChar) + ".idl.h");
    if (!file.getParentFile().exists()) file.getParentFile().mkdirs();
    final PrintStream out = new PrintStream(new FileOutputStream(file));
    out.print(cw.toString());
    out.close();

    return cw.toString();
  }
}
