/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.composite.c.components;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.components.ComponentBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the instantiation of a primitive component written in <code>C</code>
 * languages.
 */
public class CloneableInstanceBuilder implements ComponentBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the ComponentBuilder interface
  // --------------------------------------------------------------------------

  public Object createComponent(final Object componentType,
      final Object componentInstance, final Object controllerDesc,
      final Object contentDesc, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {
    final CodeWriter cw = new CodeWriter(
        "Cloneable Composite Instantiation Builder");
    final String instanceName = componentInstance.toString().replace('.', '_');

    // Declare the component structure since it may be referenced by the
    // controller structures.
    // This declaration is extern since it is redeclared and initialized latter.
    // This avoid warnings when compiling with -Wredundant-decls.
    cw.append("extern struct __cecilia_cloneableCompositeData_t ").append(
        instanceName).append(";").endl();

    cw.append((String) codePieces.get("controller"));

    cw.append("struct __cecilia_cloneableCompositeData_t ")
        .append(instanceName).append(" = {").endl();
    cw.append("{").endl();

    cw.appendln("// Server interfaces");
    cw.append("{&__cecilia_cloneableComposite_ComponentMeths, &").append(
        instanceName).append("}, // myreference").endl();
    cw.append("{&__cecilia_cloneableComposite_BindingControllerMeths, &")
        .append(instanceName).append("}, // bc").endl();
    cw.append("{&__cecilia_defaultComposite_LifeCycleControllerMeths, &(")
        .append(instanceName).append(".defaultComposite)}, ").append(
            "// lcc: use the default composite implementation").endl();
    cw.append("{&__cecilia_defaultComposite_ContentControllerMeths, &(")
        .append(instanceName).append(".defaultComposite)}, ").append(
            "// cc: use the default composite implementation").endl();

    cw.appendln("// private data");
    cw.append("sizeof(").append(instanceName).append(
        "_subComponents) / sizeof(").append(instanceName).append(
        "_subComponents[0]), // nbSubComponents").endl();
    cw.append("sizeof(").append(instanceName).append("_bindings) / sizeof(")
        .append(instanceName).append("_bindings[0]), // nbBindings").endl();
    cw.append("0, // started").endl();
    cw.append(instanceName).append("_subComponents,").endl();
    cw.append(instanceName).append("_bindings,").endl();
    cw.appendln("},");

    cw.append("{&__cecilia_cloneableComposite_FactoryMeths, &").append(
        instanceName).append("},").endl();
    cw.append("0 // factory-allocator client interface").endl();
    cw.appendln("};");

    return cw.toString();
  }
}
