/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author:Ali Erdem Ozcan
 */

package org.objectweb.fractal.cecilia.composite.c.controllers;

import java.util.Map;

import org.objectweb.fractal.cecilia.adl.bindings.BindingBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Binding builder.
 */
public class BCCompositeInstanceBuilder implements BindingBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the BindingBuilder interface
  // --------------------------------------------------------------------------

  public Object bindComponent(final Object componentType,
      final Object componentInstance, final int bindingType,
      final Object client, final String clientItf, final Object server,
      final String serverItf, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {

    if (client == null) {
      final CodeWriter cw = new CodeWriter(
          "Composite Binding Controller Initialization Builder");
      final String instanceName = componentInstance.toString()
          .replace('.', '_');
      cw.append("struct BindingDesc_t ").append(instanceName).append(
          "_bindings[] = {").endl();
      for (final Object codePiece : codePieces.values()) {
        cw.append(codePiece.toString()).append(",").endl();
      }
      cw.append("};").endl();

      return cw.toString();
    }

    final CodeWriter cw = new CodeWriter();

    final String clientIdx;
    final String serverIdx;
    if ((bindingType & BindingBuilder.NEI_BINDING_MASK) == BindingBuilder.IMPORT_BINDING)
      serverIdx = "COMPOSITE_IDX";
    else
      serverIdx = server.toString().replace('.', '_') + "_exp";

    if ((bindingType & BindingBuilder.NEI_BINDING_MASK) == BindingBuilder.EXPORT_BINDING)
      clientIdx = "COMPOSITE_IDX";
    else
      clientIdx = client.toString().replace('.', '_') + "_exp";

    cw.append("{").endl();
    cw.append(serverIdx).append(", // server").endl();
    cw.append(clientIdx).append(", // client").endl();
    if ((bindingType & BindingBuilder.STATIC_BINDING_MASK) == BindingBuilder.STATIC_BINDING)
      cw.append("BINDING_BOUND");
    else {
      cw.append("BINDING_UNBOUND");
    }
    if ((bindingType & BindingBuilder.START_PROPERTY_MASK) == BindingBuilder.AFTERSTART_BINDING)
      cw.append(" | BINDING_BIND_AFTER_START");
    else if ((bindingType & BindingBuilder.START_PROPERTY_MASK) == BindingBuilder.USED_AT_START)
      cw.append(" | BINDING_USED_AT_START");
    else
      cw.append(" | BINDING_DEFAULT_START_PROPERTY");
    cw.append(", // state").endl();

    cw.append("\"").append(serverItf).append("\", // serverName").endl();
    cw.append("\"").append(clientItf).append("\" // clientName").endl();
    cw.append("}");
    return cw.toString();

  }
}
