/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 * Contributor: Alessio Pace
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.controllers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.attributes.Attribute;
import org.objectweb.fractal.cecilia.adl.attributes.AttributeBuilder;
import org.objectweb.fractal.cecilia.adl.idl.ast.Field;
import org.objectweb.fractal.cecilia.adl.idl.ast.FieldContainer;
import org.objectweb.fractal.cecilia.adl.idl.ast.Type;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.cecilia.adl.idl.util.Util;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Build the instantiation if the internal data of the attribute controller.
 */
public class ACInstanceBuilder implements AttributeBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the AttributesBuilder interface
  // --------------------------------------------------------------------------

  public Object setAttributes(final Object componentType,
      final Object componentInstance, final Object attributeAST,
      final Attribute[] fields, final Map<String, Object> codePieces,
      final Map<Object, Object> context) throws Exception {
    final CodeWriter cw = new CodeWriter(
        "Attribute Controller Instantiation Builder");
    final String instanceName = componentInstance.toString().replace('.', '_');
    cw.append("struct ").append(componentType.toString().replace('.', '_'))
        .append("_attributedskeleton ").append(instanceName).append(
            "_attributes = {").append(fields.length).append(", {{").endl();

    final Field[] recordFields = ((FieldContainer) attributeAST).getFields();
    for (final Attribute field : fields) {
      Type attType = null;
      for (final Field fieldDefinition : recordFields) {
        if (field.getName().equals(fieldDefinition.getName())) {
          attType = Util.getContainedType(fieldDefinition);
          break;
        }
      }
      if (attType == null) {
        throw new ADLException(
            "The attribute record does not defines a field named '"
                + field.getName() + "'", (Node) attributeAST);
      }

      cw.append("\"").append(field.getName()).append("\", &").append(
          instanceName).append(".attributes.").append(field.getName()).append(
          ", sizeof(");

      cw.append(CUtil.buildComplexOrPrimitiveType(attType)).append("),").endl();

    }
    cw.append("}}").endl();

    /*
     * these informations are related to constant fields which should not be
     * allowed to be modified by the usage of the AttributeController interface.
     */
    final List<Field> constFields = getConstFields(recordFields);
    if (!constFields.isEmpty()) {
      cw.append(",");
      cw.append(constFields.size() + ", ");
      cw.append("{");
      for (final Field f : constFields) {
        cw.append("\"" + f.getName() + "\",");
      }
      cw.append("}");
    }
    cw.append("};"); // terminate attribute data static declaration
    return cw.toString();
  }

  protected List<Field> getConstFields(final Field[] fields) {
    final List<Field> result = new ArrayList<Field>();
    for (final Field f : fields) {
      if (f.getQualifier().equals(Field.CONST)) {
        result.add(f);
      }
    }
    return result;
  }
}
