/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.source;

import java.io.File;
import java.util.Map;

import org.objectweb.fractal.cecilia.adl.source.AbstractSourceFileBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Creates new generated source file for primitive component written in
 * <code>macro-c</code> and <code>manual-c</code> languages.
 */
public class SourceFileBuilder extends AbstractSourceFileBuilder {

  private static final String FILE_NAME_SUFFIX = ".adl.c";

  @Override
  protected String getFileName(final Object componentType,
      final Object componentInstance, final Map<Object, Object> context) {
    return componentType.toString().replace('.', File.separatorChar)
        + FILE_NAME_SUFFIX;
  }

  @Override
  protected String getFileContent(final Object componentType,
      final Object componentInstance, final Map<String, Object> codePieces,
      final Map<Object, Object> context) {
    final CodeWriter cw = new CodeWriter();

    final String def = (String) codePieces.get("definition");
    if (def == null)
      throw new IllegalArgumentException(
          "Cannot find the \"definition\" code piece.");
    cw.appendln(def);

    cw.appendln((String) codePieces.get("implementation"));
    cw.appendln((String) codePieces.get("factory"));

    final String instances = (String) codePieces.get("instances");
    if (instances == null)
      throw new IllegalArgumentException(
          "Cannot find the \"instances\" code piece.");
    cw.appendln(instances);

    return cw.toString();
  }
}
