/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.primitive.thinkMC.types;

import static org.objectweb.fractal.cecilia.adl.controllers.ControllerDecorationUtil.getDataSuffix;
import static org.objectweb.fractal.cecilia.adl.controllers.ControllerDecorationUtil.getVTable;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the instantiation of the exported interfaces of a component written in
 * <code>thinkMC</code> languages.
 */
public class ExportedInstanceBuilder implements InterfaceBuilder {

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context)
      throws Exception {

    final Map<String, String> producedCode = new HashMap<String, String>();
    final CodeWriter cw = new CodeWriter(
        "Exported Interfaces Instantiation Builder");
    final String definitionName = componentType.toString().replace('.', '_');
    final String instanceName = componentInstance.toString().replace('.', '_');

    cw.append("{").endl();

    for (final TypeInterface itf : interfaceTypes) {
      cw.append("{&");

      // If there is a special close for the implementation vft, we apply it
      final String vTable = getVTable(itf);
      if (vTable != null) {
        // If the interface name is given by a controller (ex:
        // _bindingcontrollermeth)
        cw.append(vTable);
      } else {
        // If this is a normal interface (VFT interfaces)
        cw.append(definitionName).append("_").append(
            itf.getName().replace('-', '_')).append("meth");
      }
      cw.append(", &(").append(instanceName);
      final String data = getDataSuffix(itf);
      if (data != null) cw.append('_').append(data);

      cw.append(")").endl();
      /* Add the signature */
      cw.append("#if defined(TYPEDINTERFACE)").endl();
      cw.append(", \"").append(itf.getSignature()).append("\"").endl();
      cw.append("#endif").endl();

      cw.append("},").endl();
    }
    cw.append("}").endl();

    producedCode.put("init", cw.toString());
    return producedCode;
  }
}
